#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Clients.OKB.Authentication;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Optimization;

namespace HeuristicLab.Clients.OKB.Administration {
  [View("Algorithm View")]
  [Content(typeof(Algorithm), true)]
  public partial class AlgorithmView : NamedOKBItemView {
    private List<Platform> platformComboBoxValues;
    private List<AlgorithmClass> algorithmClassComboBoxValues;
    private TypeSelectorDialog typeSelectorDialog;

    public new Algorithm Content {
      get { return (Algorithm)base.Content; }
      set { base.Content = value; }
    }

    public AlgorithmView() {
      InitializeComponent();
    }

    protected override void Dispose(bool disposing) {
      if (disposing) {
        if (components != null) components.Dispose();
        if (typeSelectorDialog != null) typeSelectorDialog.Dispose();
      }
      base.Dispose(disposing);
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();

      platformComboBoxValues = AdministrationClient.Instance.Platforms.ToList();
      platformComboBox.DataSource = platformComboBoxValues;
      algorithmClassComboBoxValues = AdministrationClient.Instance.AlgorithmClasses.ToList();
      algorithmClassComboBox.DataSource = algorithmClassComboBoxValues;

      if (Content == null) {
        platformComboBox.SelectedIndex = -1;
        algorithmClassComboBox.SelectedIndex = -1;
        dataTypeNameTextBox.Text = string.Empty;
        dataTypeTypeNameTextBox.Text = string.Empty;
      } else {
        platformComboBox.SelectedItem = platformComboBoxValues.FirstOrDefault(p => p.Id == Content.PlatformId);
        algorithmClassComboBox.SelectedItem = algorithmClassComboBoxValues.FirstOrDefault(a => a.Id == Content.AlgorithmClassId);
        dataTypeNameTextBox.Text = Content.DataTypeName;
        dataTypeTypeNameTextBox.Text = Content.DataTypeTypeName;
      }
      usersListBox.DataSource = null;
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      platformComboBox.Enabled = (Content != null) && !ReadOnly;
      algorithmClassComboBox.Enabled = (Content != null) && !ReadOnly;
      dataTypeGroupBox.Enabled = (Content != null) && !ReadOnly;
      refreshUsersButton.Enabled = (Content != null) && (Content.Id != 0);
      storeUsersButton.Enabled = (usersListBox.DataSource != null) && !ReadOnly;
      usersListBox.Enabled = (usersListBox.DataSource != null) && !ReadOnly;

      bool isHL33Platform = platformComboBox.Text == "HeuristicLab 3.3";
      setDataTypeButton.Enabled = isHL33Platform && !ReadOnly;
      dataTypeNameTextBox.ReadOnly = isHL33Platform;
      dataTypeTypeNameTextBox.ReadOnly = isHL33Platform;
    }

    protected override void OnContentPropertyChanged(string propertyName) {
      switch (propertyName) {
        case "Id":
          SetEnabledStateOfControls();
          break;
        case "PlatformId":
          platformComboBox.SelectedItem = platformComboBoxValues.FirstOrDefault(p => p.Id == Content.PlatformId);
          SetEnabledStateOfControls();
          break;
        case "AlgorithmClassId":
          algorithmClassComboBox.SelectedItem = algorithmClassComboBoxValues.FirstOrDefault(a => a.Id == Content.AlgorithmClassId);
          break;
        case "DataTypeName":
          dataTypeNameTextBox.Text = Content.DataTypeName;
          break;
        case "DataTypeTypeName":
          dataTypeTypeNameTextBox.Text = Content.DataTypeTypeName;
          break;
      }
    }

    private void platformComboBox_SelectedValueChanged(object sender, System.EventArgs e) {
      if (Content != null) {
        Platform selected = platformComboBox.SelectedItem as Platform;
        if (selected != null) Content.PlatformId = selected.Id;
      }
    }
    private void algorithmClassComboBox_SelectedValueChanged(object sender, System.EventArgs e) {
      if (Content != null) {
        AlgorithmClass selected = algorithmClassComboBox.SelectedItem as AlgorithmClass;
        if (selected != null) Content.AlgorithmClassId = selected.Id;
      }
    }

    private void setDataTypeButton_Click(object sender, EventArgs e) {
      if (typeSelectorDialog == null) {
        typeSelectorDialog = new TypeSelectorDialog();
        typeSelectorDialog.Caption = "Select Algorithm";
        typeSelectorDialog.TypeSelector.Caption = "Available Algorithms";
        typeSelectorDialog.TypeSelector.Configure(typeof(IAlgorithm), false, true);
      }
      if (typeSelectorDialog.ShowDialog(this) == DialogResult.OK) {
        Content.DataTypeName = typeSelectorDialog.TypeSelector.SelectedType.Name;
        Content.DataTypeTypeName = typeSelectorDialog.TypeSelector.SelectedType.AssemblyQualifiedName;
      }
    }
    private void dataTypeNameTextBox_TextChanged(object sender, EventArgs e) {
      if (dataTypeNameTextBox.Text != Content.DataTypeName)
        Content.DataTypeName = dataTypeNameTextBox.Text;
    }
    private void dataTypeTypeNameTextBox_TextChanged(object sender, EventArgs e) {
      if (dataTypeTypeNameTextBox.Text != Content.DataTypeTypeName)
        Content.DataTypeTypeName = dataTypeTypeNameTextBox.Text;
    }

    private void refreshUsersButton_Click(object sender, System.EventArgs e) {
      List<Guid> ids = AdministrationClient.GetAlgorithmUsers(Content.Id);
      if (ids != null) {
        if (AuthenticationClient.Instance.Users == null) AuthenticationClient.Instance.Refresh();
        List<User> users = AuthenticationClient.Instance.Users.ToList();
        usersListBox.DataSource = users;
        usersListBox.DisplayMember = "Name";
        for (int i = 0; i < users.Count; i++)
          usersListBox.SetItemChecked(i, ids.Contains(users[i].Id));
        usersListBox.Enabled = !ReadOnly;
        storeUsersButton.Enabled = false;
      }
    }
    private void storeUsersButton_Click(object sender, System.EventArgs e) {
      AdministrationClient.UpdateAlgorithmUsers(Content.Id, usersListBox.SelectedItems.Cast<User>().Select(u => u.Id).ToList());
      storeUsersButton.Enabled = false;
    }
    private void usersListBox_ItemCheck(object sender, ItemCheckEventArgs e) {
      storeUsersButton.Enabled = !ReadOnly;
    }
  }
}
